<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/

class core_handle {

	/**
	* Array that keep track of the loaded plugins.
	*
	* @var	 array
	* @access  private
	* @see	 check_object()
	*/

	/*<{php4}>
	var $_arrExtended = array();
	<{php4/}>*/

	/*<{php5}>
	public $_arrExtended = array();
	<{php5/}>*/

	/*<{delete}>*/
	var $_arrExtended = array();
	/*<{delete/}>*/

	/**
	* Configuration array for phpPlugin.
	*
	* @var	 array
	* @access  private
	*/

	/*<{php4}>
	var $_arrConfig = array();
	<{php4/}>*/

	/*<{php5}>
	public $_arrConfig = array();
	<{php5/}>*/

	/*<{delete}>*/
	var $_arrConfig = array();
	/*<{delete/}>*/

	/*<{php4}>
	var $_autoload = '';
	<{php4/}>*/

	/*<{php5}>
	public $_autoload = '';
	<{php5/}>*/

	/*<{delete}>*/
	var $_autoload = '';
	/*<{delete/}>*/

	/**
	* Contructor
	*
	* @param   string  $strPath  Sets the path to the plugins files
	* @access  public
	*/

	function core_handle ($strPath = '') {

		$this->set_path ($strPath);
		$this->set_filename ('class.?.php');

	}

	/**
	* Sets the path to the plugins files
	*
	* @param   string  $strPath  Path to the plugins files
	* @access  public
	*/

	function set_path ($strPath = '') {

		$this->_arrConfig['plugins_path'] = $strPath;

	}

	/**
	* Sets the filename format for the plugins.
	*
	* If $strClassName is not passed, the filename is default for
	* all Classes that don't set their own formats.
	*
	* A modifier function can be passed along with other parameters
	* that will transform the plugin's filename to the correct one.
	*
	* @param   string  $str		   Filename format. Characters ? will be
	*								 replaced with the plugin's name.
	*
	* @param   string  $strClassName  Class name. If not passed, sets
	*								 the default filename to all classes.
	*
	* @param   string  $strModifier   Modifier function name.
	*
	* @param   array   $arrParams	 Array of parameters for the modifier
	*								 function. The first parameter will be
	*								 the plugin's name. Should pass only
	*								 the second, third, and so on.
	*
	* @access  public
	*/

	function set_filename ($str, $strClassName = '', $strModifier = null, $arrParams = null) {
		if (empty ($strClassName) ) {
			$this->_arrConfig['filename']['default'] = array ($str,
									$strModifier,
									is_null ($arrParams)?array () : $arrParams);
		} else {
			$this->_arrConfig['filename']['classes'][$strClassName] = array ($str,
											$strModifier,
											is_null ($arrParams)?array () : $arrParams);
		}

	}

	/**
	* Loads a plugin in the object passed by $obj.
	*
	* @param   string   $strPlugin	 Plugin's name. Will be used to define
	*								  the filename.
	* @param   object   $obj		   Object that will receive the plugin.
	* @param   string   $strClassName  Class name of the object.
	* @param   boolean  $booForce	  Force reload of the plugin.
	*
	* @return  string   New class name with the extended plugin.
	* @access  public
	* @uses	_check_object()
	* @uses	load_string()
	*/

	function load ($strPlugin, &$obj, $strClassName = null, $booForce = false) {

		$boo = $this->_check_object ($strPlugin, $obj, $strClassName);
		if ($boo === false && $booForce === false) {
			return get_class ($obj);
		}
		if (isset ($this->_arrConfig['filename']['classes'][$strClassName]) ) {
			$arrTemplate = $this->_arrConfig['filename']['classes'][$strClassName];
		} else {
			$arrTemplate = $this->_arrConfig['filename']['default'];
		}
		$strFileName = str_replace ('?', $strPlugin, $arrTemplate[0]);
		if (!is_null ($arrTemplate[1]) ) {
			array_unshift ($arrTemplate[2], $strFileName);
			$strFileName = call_user_func_array ($arrTemplate[1], $arrTemplate[2]);
		}
		$this->_autoload = '_init_' . str_replace ('.', '_', $strFileName);
		$strFileName = $this->_arrConfig['plugins_path'] . $strFileName;
		if (!file_exists ($strFileName) ) {
			print '<br /><strong>File doesn not exist ' . $strFileName . '</strong><br />';
			exit;
		}
		$arrContent = file ($strFileName);
		$booContinue = true;
		$i = 0;
		while ($booContinue) {
			$i++;
			foreach ($arrContent as $intLine => $strLine) {
				$strLine = trim ($strLine);
				array_shift ($arrContent);
				if (!empty ($strLine) ) {
					break;
				}
			}
			$arrContent = array_reverse ($arrContent);
			if ($i == 2) {
				$booContinue = false;
			}
		}
		unset ($booContinue);
		$strContent = implode ("\r\n", $arrContent);
		unset ($arrContent);
		$strContent = str_replace ('class opn_placebo_name extends opn_management {', '', $strContent);
		$strContent = str_replace ('} ?>', '?>', $strContent);
		return $this->load_string ($strPlugin, $strContent, $obj, $strClassName, $booForce);

	}

	/**
	* Loads a plugin from a variable in the object passed by $obj.
	*
	* @param   string   $strPlugin	 Plugin's name. Will be used to define
	*								  the filename.
	* @param   string   $strContent	Plugin's definition content.
	* @param   object   $obj		   Object that will receive the plugin.
	* @param   string   $strClassName  Class name of the object.
	* @param   boolean  $booForce	  Force reload of the plugin.
	*
	* @return  string   New class name with the extended plugin.
	* @access  public
	* @uses	_check_object()
	* @uses	_extend()
	*/

	function load_string ($strPlugin, $strContent, &$obj, $strClassName = null, $booForce = false) {

		$boo = $this->_check_object ($strPlugin, $obj, $strClassName);
		if ($boo === false && $booForce === false) {
			return get_class ($obj);
		}
		if ($boo === true) {
			$this->_arrExtended[$strClassName][] = $strPlugin;
		}
		return $this->_extend ($obj, $strContent, $strClassName);

	}

	function highlight_code ($code) {

		$code = "\n" . $code . "\n";
		$code2 = stripslashes ($code);
		$split = split ("\n", $code2);
		$mv = count ($split);

		/*for design reasons */
		if ($mv<100) {
			$width = '15';
		} elseif ($mv<1000) {
			$width = '22';
		} elseif ($mv<10000) {
			$width = '30';
		} else {
			$width = '15';
		}
		$source_code = '<table bgcolor="#FFFFFF" cellspacing="0" width="100%">';
		$code2 = highlight_string ($code2, true);
		for ($i = 0; $i< $mv; $i++) {
			$t = $i+1;
			$source_code .= '<tr><td bgcolor="#FFFFE1" width="' . $width . '">';
			$source_code .= '<div align="right"><font face="verdana" size="1">' . $t . '</font></div></td>';
			if ($t<2) {
				$source_code .= '<td style="padding-left: 5px" rowspan="10000" valign="top">' . $code2 . '</td>';
			}
			$source_code .= '</tr>';
		}
		$source_code .= '</table>';
		return $source_code;

	}

	/**
	* Extends the class with the new plugin.
	*
	* @param   object   $obj		   Object that will receive the plugin.
	* @param   string   $strContent	Plugin's definition content.
	*
	* @return  string   New class name with the extended plugin.
	* @access  private
	*/

	function _extend (&$obj, $strContent) {

		global $opnConfig;

		$strClass2Extend = get_class ($obj);
		mt_srand ((double)microtime ()*1000000);
		$strNewClass = $strClass2Extend . mt_rand (0, 9999);
		$strContent = str_replace ("\r\n\r\n", "\r\n", $strContent);
		if (isset ($opnConfig['option']['eval_debug']) ) {
			$str = 'class ' . $strNewClass . ' extends ' . $strClass2Extend . ' { ' . "\r\n" . $strContent . ' } ';
			echo $this->highlight_code ($str);
		}
		eval ('class ' . $strNewClass . ' extends ' . $strClass2Extend . ' { ' . "\r\n" . $strContent . ' } ');
		unset ($strContent);
		$objNew = new $strNewClass ();
		$arrVars = get_class_vars ($strClass2Extend);
		foreach ($arrVars as $key => $value) {
			$objNew->{$key} = &$obj->{$key};
		}
		unset ($arrVars);
		$obj = $objNew;
		unset ($objNew);
		$dummy = $this->_autoload;
		// $obj->$dummy();
		return $strNewClass;

	}

	/**
	* Check if the plugin is already loaded from the Class passed by $strClassName.
	*
	* @param   object   $obj		   Object that will receive the plugin.
	* @param   string   $strPlugin	 Plugin's name.
	* @param   string   $strClassName  Class name of the object.
	*
	* @return  boolean  false if the plugin is already loaded, true otherwise.
	* @access  private
	*/

	function _check_object ($strPlugin, $obj, &$strClassName) {
		if (is_null ($strClassName) || empty ($strClassName) ) {
			$strClassName = get_class ($obj);
		}
		if (!isset ($this->_arrExtended[$strClassName]) ) {
			$this->_arrExtended[$strClassName] = array ();
		} else {
			$keyExt = array_search ($strPlugin, $this->_arrExtended[$strClassName]);
			if ($keyExt !== false) {
				return false;
			}
		}
		return true;

	}

}

?>