<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_OPN_SMTP_INCLUDED') ) {
	define ('_OPN_CLASS_OPN_SMTP_INCLUDED', 1);
	include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'net/class.opn_socket.php');

	class opn_smtp {

		/**
		* The server to connect to.
		* @var string
		* @access public
		*/
		var $host = 'localhost';

		/**
		* The port to connect to.
		* @var int
		* @access public
		*/
		var $port = 25;

		/**
		* The value to give when sending EHLO or HELO.
		* @var string
		* @access public
		*/
		var $localhost = 'localhost';

		/**
		* List of supported authentication methods, in preferential order.
		* @var array
		* @access public
		*/
		var $auth_methods = array ('DIGEST-MD5',
					'CRAM-MD5',
					'LOGIN',
					'PLAIN');

		/**
		* Should debugging output be enabled?
		* @var boolean
		* @access private
		*/
		var $_debug = false;

		/**
		* The socket resource being used to connect to the SMTP server.
		* @var resource
		* @access private
		*/
		var $_socket = null;

		/**
		* The most recent server response code.
		* @var int
		* @access private
		*/
		var $_code = -1;

		/**
		* The most recent server response arguments.
		* @var array
		* @access private
		*/
		var $_arguments = array ();

		/**
		* Stores detected features of the SMTP server.
		* @var array
		* @access private
		*/
		var $_esmtp = array ();

		/**
		* FUNCTION: opn_smtp
		* Instantiates a new opn_smtp object, overriding any defaults
		* with parameters that are passed in.
		*
		* @param string $host The server to connect to.
		* @param integer $port The port to connect to.
		* @param string $localhost The value to give when sending EHLO or HELO.
		* @access public
		*/

		function opn_smtp ($host = null, $port = null, $localhost = null) {
			if (isset ($host) ) {
				$this->host = $host;
			}
			if (isset ($port) ) {
				$this->port = $port;
			}
			if (isset ($localhost) ) {
				$this->localhost = $localhost;
			}
			$this->_socket = new opn_socket ();
			if ( (@include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'auth/class.opn_auth.php') ) === false) {
				$pos = array_search ('DIGEST-MD5', $this->auth_methods);
				unset ($this->auth_methods[$pos]);
				$pos = array_search ('CRAM-MD5', $this->auth_methods);
				unset ($this->auth_methods[$pos]);
			}

		}
		// end function

		/**
		* FUNCTION: setDebug
		* Set the value of the debugging flag.
		*
		* @param   boolean $debug	  New value for the debugging flag.
		* @access public
		*/

		function setDebug ($debug) {

			$this->_debug = $debug;

		}
		// end function

		/**
		* FUNCTION: getResponseSMTP
		* Return a 2-tuple containing the last response from the SMTP server.
		*
		* @return  array   A two-element array: the first element contains the
		* response code as an integer and the second element contains the
		* response's arguments as a string.
		* @access public
		*/

		function getResponseSMTP () {
			return array ($this->_code,
					join ("\n",
					$this->_arguments) );

		}
		// end function

		/**
		* FUNCTION: connect
		* Attempt to connect to the SMTP server.
		*
		* @param   int	 $timeout	The timeout value (in seconds) for the
		* socket connection.
		* @param   bool	$persistent Should a persistent socket connection be
		* used?
		* @return bool Returns false of failure, or true on success.
		* @access public
		*/

		function connectsmtp ($timeout = null, $persistent = false) {

			$result = $this->_socket->connectsocket ($this->host, $this->port, $persistent, $timeout);
			if (!$result) {
				opnErrorHandler (E_WARNING, 'Failed to connect socket', __FILE__, __LINE__);
				return false;
			}
			$result = $this->_parseResponse (220);
			if (!$result) {
				opnErrorHandler (E_WARNING, 'Wrong response', __FILE__, __LINE__);
				return false;
			}
			$result = $this->_negotiate ();
			if (!$result) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: disconnect
		* Attempt to disconnect from the SMTP server.
		*
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function disconnect () {
			if (!$this->_put ('QUIT') ) {
				return false;
			}
			if (!$this->_parseResponse (221) ) {
				return false;
			}
			if (!$this->_socket->disconnect () ) {
				opnErrorHandler (E_WARNING, 'Failed to disconnect socket', __FILE__, __LINE__);
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: auth
		* Attempt to do SMTP authentication.
		*
		* @param string The userid to authenticate as.
		* @param string The password to authenticate with.
		* @param string The requested authentication method. If none is
		* specified, the best supported method will be used.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		*/

		function auth ($uid, $pwd, $method = '') {
			if (empty ($this->_esmtp['AUTH']) ) {
				opnErrorHandler (E_WARNING, 'SMTP server does no support authentication', __FILE__, __LINE__);
				return false;
			}

			/*
			* If no method has been specified, get the name of the best supported
			* method advertised by the SMTP server.
			*/
			if (empty ($method) ) {
				$method = $this->_getBestAuthMethod ();
				if ($method === false) {

					/* Return the PEAR_Error object from _getBestAuthMethod(). */
					return false;
				}
			} else {
				$method = strtoupper ($method);
				if (!in_array ($method, $this->auth_methods) ) {
					opnErrorHandler (E_WARNING, $method . ' is not a supported authentication method', __FILE__, __LINE__);
					return false;
				}
			}
			switch ($method) {
				case 'DIGEST-MD5':
					$result = $this->_authDigest_MD5 ($uid, $pwd);
					break;
				case 'CRAM-MD5':
					$result = $this->_authCRAM_MD5 ($uid, $pwd);
					break;
				case 'LOGIN':
					$result = $this->_authLogin ($uid, $pwd);
					break;
				case 'PLAIN':
					$result = $this->_authPlain ($uid, $pwd);
					break;
				default:
					opnErrorHandler (E_WARNING, $method . ' is not a supported authentication method', __FILE__, __LINE__);
					break;
			}

			/* If an error was encountered, return the PEAR_Error object. */
			if ($result === false) {
				return false;
			}

			/* RFC-2554 requires us to re-negotiate ESMTP after an AUTH. */
			if (!$this->_negotiate () ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: helo
		* Send the HELO command.
		*
		* @param string $domain The domain name to say we are.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function helo ($domain) {
			if (!$this->_put ('HELO', $domain) ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: mailFrom
		* Send the MAIL FROM: command.
		*
		* @param string $sender The sender (reverse path) to set.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function mailFrom ($sender) {
			if (!$this->_put ('MAIL', 'FROM:<' . $sender . '>') ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: rcptTo
		* Send the RCPT TO: command.
		*
		* @param string $recipient The recipient (forward path) to add.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function rcptTo ($recipient) {
			if (!$this->_put ('RCPT', "TO:<$recipient>") ) {
				return false;
			}
			$a = array (250, 251);
			if (!$this->_parseResponse ($a) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: quotedata
		* This is provided as a separate public function to facilitate easier
		* overloading for the cases where it is desirable to customize the
		* quoting behavior.
		*
		* @param string $data The message text to quote.  The string must be
		* passed by reference, and the text will be modified in place.
		* @access public
		*/

		function quotedata (&$data) {

			/*
			* Change Unix (\n) and Mac (\r) linefeeds into Internet-standard CRLF
			* (\r\n) linefeeds.
			*/

			$data = preg_replace ("/([^\r]{1})\n/", "\\1\r\n", $data);
			$data = preg_replace ("/\n\n/", "\n\r\n", $data);

			/*
			* Because a single leading period (.) signifies an end to the data,
			* legitimate leading periods need to be "doubled" (e.g. '..').
			*/

			$data = preg_replace ("/\n\./", "\n..", $data);

		}
		// end function

		/**
		* FUNCTION: data
		* Send the DATA command.
		*
		* @param string $data The message body to send.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function data ($data) {

			/*
			* RFC 1870, section 3, subsection 3 states "a value of zero indicates
			* that no fixed maximum message size is in force".  Furthermore, it
			* says that if "the parameter is omitted no information is conveyed
			* about the server's fixed maximum message size".
			*/
			if (isset ($this->_esmtp['SIZE']) && ($this->_esmtp['SIZE']>0) ) {
				if (strlen ($data) >= $this->_esmtp['SIZE']) {
					$this->disconnect ();
					opnErrorHandler (E_WARNING, 'Message size excedes the server limit', __FILE__, __LINE__);
					return false;
				}
			}

			/* Quote the data based on the SMTP standards. */

			$this->quotedata ($data);
			if (!$this->_put ('DATA') ) {
				return false;
			}
			if (!$this->_parseResponse (354) ) {
				return false;
			}
			if (!$this->_send ($data . "\r\n.\r\n") ) {
				opnErrorHandler (E_WARNING, 'write to socket failed', __FILE__, __LINE__);
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: sendFrom
		* Send the SEND FROM: command.
		*
		* @param string $path The reverse path to send.
		* @return mixed Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function sendFrom ($path) {
			if (!$this->_put ('SEND', "FROM:<$path>") ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: somlFrom
		* Send the SOML FROM: command.
		*
		* @param string $path The reverse path to send.
		* @return mixed Returns false on any
		*			   kind of failure, or true on success.
		* @access public
		*/

		function somlFrom ($path) {
			if (!$this->_put ('SOML', "FROM:<$path>") ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: samlFrom
		* Send the SAML FROM: command.
		*
		* @param string $path The reverse path to send.
		* @return mixed Returns false on any
		*			   kind of failure, or true on success.
		* @access public
		*/

		function samlFrom ($path) {
			if (!$this->_put ('SAML', "FROM:<$path>") ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: rset
		* Send the RSET command.
		*
		* @return mixed Returns false on any
		*			   kind of failure, or true on success.
		* @access public
		*/

		function rset () {
			if (!$this->_put ('RSET') ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: vrfy
		* Send the VRFY command.
		*
		* @param string $string The string to verify.
		* @return mixed Returns false on any
		*			   kind of failure, or true on success.
		* @access public
		*/

		function vrfy ($string) {

			/* Note: 251 is also a valid response code */
			if (!$this->_put ('VRFY', $string) ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: noop
		* Send the NOOP command.
		*
		* @return mixed Returns false on any
		*			   kind of failure, or true on success.
		* @access public
		*/

		function noop () {
			if (!$this->_put ('NOOP') ) {
				return false;
			}
			if (!$this->_parseResponse (250) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: _send
		* Send the given string of data to the server.
		*
		* @param   string  $data	   The string of data to send.
		* @return  bool   True on success or false on failure.
		*/

		function _send ($data) {
			if ($this->_debug) {
				echo 'DEBUG: Send: ' . htmlentities ($data) . '<br />';
			}
			$result = $this->_socket->write ($data);
			return $result;

		}
		// end function

		/**
		* FUNCTION: _put
		* Send a command to the server with an optional string of arguments.
		* A carriage return / linefeed (CRLF) sequence will be appended to each
		* command string before it is sent to the SMTP server.
		*
		* @param   string  $command	The SMTP command to send to the server.
		* @param   string  $args	   A string of optional arguments to append
		* to the command.
		* @return  bool   The result of the _send() call.
		* @access private
		*/

		function _put ($command, $args = '') {
			if (!empty ($args) ) {
				return $this->_send ($command . ' ' . $args . "\r\n");
			}
			return $this->_send ($command . "\r\n");

		}
		// end function

		/**
		* FUNCTION: _parseResponse
		* Read a reply from the SMTP server.  The reply consists of a response
		* code and a response message.
		*
		* @param   mixed   $valid	  The set of valid response codes. These
		* may be specified as an array of integer values or as a single integer
		* value.
		* @return  mixed   True if the server returned a valid response code or
		* false is an error condition is reached.
		* @access private
		* @see getResponse
		*/

		function _parseResponse ($valid) {

			$this->_code = -1;
			$this->_arguments = array ();
			while ($line = $this->_socket->readLine () ) {
				if ($this->_debug) {
					echo 'DEBUG: Recv: ' . htmlentities ($line) . '<br />';
				}

				/* If we receive an empty line, the connection has been closed. */
				if (empty ($line) ) {
					$this->disconnect ();
					opnErrorHandler (E_WARNING, 'Connection was unexpectedly closed', __FILE__, __LINE__);
					return false;
				}

				/* Read the code and store the rest in the arguments array. */

				$code = substr ($line, 0, 3);
				$this->_arguments[] = trim (substr ($line, 4) );

				/* Check the syntax of the response code. */
				if (is_numeric ($code) ) {
					$this->_code = (int) $code;
				} else {
					$this->_code = -1;
					break;
				}

				/* If this is not a multiline response, we're done. */
				if (substr ($line, 3, 1) != '-') {
					break;
				}
			}

			/* Compare the server's response code with the valid code. */
			if (is_int ($valid) && ($this->_code === $valid) ) {
				return true;
			}

			/* If we were given an array of valid response codes, check each one. */
			if (is_array ($valid) ) {
				foreach ($valid as $valid_code) {
					if ($this->_code === $valid_code) {
						return true;
					}
				}
			}
			return false;

		}
		// end function

		/**
		* FUNCTION: _negotiate
		* Attempt to send the EHLO command and obtain a list of ESMTP
		* extensions available, and failing that just send HELO.
		*
		* @return mixed Returns false on any kind of failure, or true on
		* success.
		* @access public
		*/

		function _negotiate () {
			if (!$this->_put ('EHLO', $this->localhost) ) {
				opnErrorHandler (E_WARNING, 'EHELO was not accepted', __FILE__, __LINE__);
				return false;
			}
			if (!$this->_parseResponse (250) ) {

				/* If we receive a 503 response, we're already authenticated. */
				if ($this->_code === 503) {
					return true;
				}

				/* If the EHLO failed, try the simpler HELO command. */
				if (!$this->_put ('HELO', $this->localhost) ) {
					opnErrorHandler (E_WARNING, 'HELO was not accepted', __FILE__, __LINE__);
					return false;
				}
				if (!$this->_parseResponse (250) ) {
					opnErrorHandler (E_WARNING, 'HELO was not accepted', __FILE__, __LINE__);
					return false;
				}
				return true;
			}
			$this->_parseESMTP ();
			return true;

		}
		// end function

		/**
		* FUNCTION: _parseESMTP
		* Parses the SMTP Server EHELO or HELO message.
		*
		* @access private
		*/

		function _parseESMTP () {

			foreach ($this->_arguments as $argument) {
				$verb = strtok ($argument, ' ');
				$arguments = substr ($argument, strlen ($verb)+1, strlen ($argument)-strlen ($verb)-1);
				$this->_esmtp[$verb] = $arguments;
			}

		}
		// end function

		/**
		* FUNCTION: _getBestAuthMethod
		* Returns the name of the best authentication method that the server
		* has advertised.
		*
		* @return mixed	Returns a string containing the name of the best
		* supported authentication method or false if a failure condition is
		* encountered.
		* @access private
		*/

		function _getBestAuthMethod () {

			$available_methods = explode (' ', $this->_esmtp['AUTH']);
			foreach ($this->auth_methods as $method) {
				if (in_array ($method, $available_methods) ) {
					return $method;
				}
			}
			opnErrorHandler (E_WARNING, 'No supported authentication methods', __FILE__, __LINE__);
			return false;

		}
		// end function

		/**
		* FUNCTION: _authDigest_MD5
		* Authenticates the user using the DIGEST-MD5 method.
		*
		* @param string The userid to authenticate as.
		* @param string The password to authenticate with.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access private
		*/

		function _authDigest_MD5 ($uid, $pwd) {
			if (!$this->_put ('AUTH', 'DIGEST-MD5') ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {

				/* 503: Error: already authenticated */
				if ($this->_code === 503) {
					return true;
				}
				return false;
			}
			$challenge = base64_decode ($this->_arguments[0]);
			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'auth/class.opn_auth_digestmd5.php');
			$digest =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_auth_digestmd5;
			$auth_str = base64_encode ($digest->getResponse ($uid, $pwd, $challenge, $this->host, "smtp") );
			if (!$this->_put ($auth_str) ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {
				return false;
			}

			/*
			* We don't use the protocol's third step because SMTP doesn't allow
			* subsequent authentication, so we just silently ignore it.
			*/
			if (!$this->_put (' ') ) {
				return false;
			}

			/* 235: Authentication successful */
			if (!$this->_parseResponse (235) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: _authCRAM_MD5
		* Authenticates the user using the CRAM-MD5 method.
		*
		* @param string The userid to authenticate as.
		* @param string The password to authenticate with.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access private
		*/

		function _authCRAM_MD5 ($uid, $pwd) {
			if (!$this->_put ('AUTH', 'CRAM-MD5') ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {

				/* 503: Error: already authenticated */
				if ($this->_code === 503) {
					return true;
				}
				return false;
			}
			$challenge = base64_decode ($this->_arguments[0]);
			include_once (_OPN_ROOT_PATH . _OPN_CLASS_SOURCE_PATH . 'auth/class.opn_auth_crammd5.php');
			$cram =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ opn_auth_crammd5;
			$auth_str = base64_encode ($cram->getResponse ($uid, $pwd, $challenge) );
			if (!$this->_put ($auth_str) ) {
				return false;
			}

			/* 235: Authentication successful */
			if (!$this->_parseResponse (235) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: _authLogin
		* Authenticates the user using the LOGIN method.
		*
		* @param string The userid to authenticate as.
		* @param string The password to authenticate with.
		* @return bool Returns false on any kind of failure, or true on
		* success.
		* @access private
		*/

		function _authLogin ($uid, $pwd) {
			if (!$this->_put ('AUTH', 'LOGIN') ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {

				/* 503: Error: already authenticated */
				if ($this->_code === 503) {
					return true;
				}
				return false;
			}
			if (!$this->_put (base64_encode ($uid) ) ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {
				return false;
			}
			if (!$this->_put (base64_encode ($pwd) ) ) {
				return false;
			}

			/* 235: Authentication successful */
			if (!$this->_parseResponse (235) ) {
				return false;
			}
			return true;

		}
		// end function

		/**
		* FUNCTION: _authPlain
		* Authenticates the user using the PLAIN method.
		*
		* @param string The userid to authenticate as.
		* @param string The password to authenticate with.
		* @return mixed Returns false on any kind of failure, or true on
		* success.
		* @access private
		*/

		function _authPlain ($uid, $pwd) {
			if (!$this->_put ('AUTH', 'PLAIN') ) {
				return false;
			}

			/* 334: Continue authentication request */
			if (!$this->_parseResponse (334) ) {

				/* 503: Error: already authenticated */
				if ($this->_code === 503) {
					return true;
				}
				return false;
			}
			$auth_str = base64_encode (chr (0) . $uid . chr (0) . $pwd);
			if (!$this->_put ($auth_str) ) {
				return false;
			}

			/* 235: Authentication successful */
			if (!$this->_parseResponse (235) ) {
				return false;
			}
			return true;

		}
		// end function

	}
	// end class
}

?>