<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_DRIVERIBASE_INCLUDED') ) {
	define ('_OPN_CLASS_DRIVERIBASE_INCLUDED', 1);

	/**
	* Extends the SQL Connection class for Interbase Database access
	*/

	class SQLLayer_ibase extends SQLConnection {
		var $_thissql = '';
		var $databaseType = 'ibase';
		var $dataProvider = 'ibase';
		var $replaceQuote = "''";
		// string to use to replace quotes
		var $_transactionID;
		var $metaTablesSQL = 'SELECT rdb$relation_name FROM rdb$relations WHERE rdb$relation_name not like \'RDB$%\'';
		var $metaColumnsSQL = 'SELECT a.rdb$field_name, a.rdb$null_flag, a.rdb$default_source, b.rdb$field_length, b.rdb$field_scale, b.rdb$field_sub_type, b.rdb$field_precision, b.rdb$field_type from rdb$relation_fields a, rdb$fields b WHERE a.rdb$field_source = b.rdb$field_name AND a.rdb$relation_name = \'%s\' ORDER BY a.rdb$field_position asc';
		var $ibasetrans;
		var $hasGenID = true;
		var $_bindInputArray = true;
		var $isInterbase = true;
		var $buffers = 0;
		var $dialect = 1;
		var $ansiOuter = true;
		var $blobEncodeType = 'C';

		/**
		* Constructor for SQLLayer_ibase
		*
		* @access public
		* @return void
		*/

		function SQLLayer_ibase () {
			if (!extension_loaded ('interbase') ) {
				trigger_error ('You must have the interbase extension installed.', E_USER_ERROR);
			}
			$this->fetchMode = true;

		}

		/**
		* Sets the database dialect.
		*
		* @access public
		* @param int	$dialect
		* @return void
		*/

		function SetDialect ($dialect) {

			$this->dialect = $dialect;

		}

		/**
		* Connect to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @param bool		$persist
		* @return bool		success
		*/

		function _connect ($argHostname, $argUsername, $argPassword, $argDatabasename, $persist = false) {
			if ($argDatabasename) {
				$argHostname .= ':' . $argDatabasename;
			}
			$fn = ($persist)?'ibase_pconnect' : 'ibase_connect';
			$this->_connectionID = $fn ($argHostname, $argUsername, $argPassword, $this->charSet, $this->buffers, $this->dialect);
			if ($this->dialect != 1) {
				$this->replaceQuote = "''";
			}
			if ($this->_connectionID === false) {
				return false;
			}
			return true;

		}

		/**
		* Persitent connection to the database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return bool		success
		*/

		function _pconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {
			return $this->_connect ($argHostname, $argUsername, $argPassword, $argDatabasename, true);

		}

		/**
		* returns the last error number from previous database operation
		*
		* @access public
		* @return int
		*/

		function ErrorNo () {
			$arr = '';
			if (version_compare (phpversion (), '5.0.0', 'ge') == 1) {
				$this->_errorCode = @ibase_errcode ();
			} elseif (preg_match ('/error code = ([\-0-9]*)/i', @ibase_errmsg (), $arr) ) {
				$this->_errorCode = (integer) $arr[1];
			} else {
				$this->_errorCode = 0;
			}
			return $this->_errorCode;

		}

		/**
		* returns the last error message from previous database operation
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {

			$this->_errorMsg = @ibase_errmsg ();
			return $this->_errorMsg;

		}

		/**
		*
		* @access public
		* @param string	$table
		* @param boolean	$owner_notused
		* @param boolean	$internalKey
		* @return mixed boolean or array
		*/

		function MetaPrimaryKeys ($table, $owner_notused = false, $internalKey = false) {

			$t = $owner_notused;
			if ($internalKey) {
				return array ('RDB$DB_KEY');
			}
			$table = strtoupper ($table);
			$sql = 'SELECT S.RDB$FIELD_NAME AFIELDNAME FROM RDB$INDICES I ' . 'JOIN RDB$INDEX_SEGMENTS S ON I.RDB$INDEX_NAME=S.RDB$INDEX_NAME ' . 'WHERE I.RDB$RELATION_NAME=\'' . $table . '\' ' . 'AND I.RDB$INDEX_NAME LIKE \'RDB$PRIMARY%\' ' . 'ORDER BY I.RDB$INDEX_NAME,S.RDB$FIELD_POSITION';
			$a = $this->GetCol ($sql, false, true);
			if ($a && sizeof ($a)>0) {
				return $a;
			}
			return false;

		}

		/**
		* Start a transaction
		*
		* @access public
		* @return mixed unknown or boolean
		*/

		function BeginTrans () {
			if ($this->transOff) {
				return true;
			}
			$this->transCnt += 1;
			$this->autoCommit = false;
			$this->_transactionID = ibase_trans ($this->ibasetrans, $this->_connectionID);
			return $this->_transactionID;

		}

		/**
		* Commit a transaction
		*
		* @access public
		* @param boolean	$ok
		* @return boolean
		*/

		function CommitTrans ($ok = true) {
			if (!$ok) {
				return $this->RollbackTrans ();
			}
			if ($this->transOff) {
				return true;
			}
			if ($this->transCnt) {
				$this->transCnt -= 1;
			}
			$ret = false;
			$this->autoCommit = true;
			if ($this->_transactionID) {
				$ret = ibase_commit ($this->_transactionID);
			}
			$this->_transactionID = false;
			return $ret;

		}

		/**
		* Rollback a transaction.
		*
		* @access public
		* @return mixed bool or boolean
		*/

		function RollbackTrans () {
			if ($this->transOff) {
				return true;
			}
			if ($this->transCnt) {
				$this->transCnt -= 1;
			}
			$ret = false;
			$this->autoCommit = true;
			if ($this->_transactionID) {
				$ret = ibase_rollback ($this->_transactionID);
			}
			$this->_transactionID = false;
			return $ret;

		}

		/**
		* Get the tableindices.
		*
		* @access public
		* @param string	$table
		* @param bool		$primary (Optional)
		* @param mixed		$owner	 (Optional)
		* @return mixed boolean or array
		*/

		function &MetaIndexes ($table, $primary = false, $owner = false) {

			$t = $owner;
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$table = strtoupper ($table);
			$sql = 'SELECT * FROM RDB$INDICES WHERE RDB$RELATION_NAME = \'' . $table . '\'';
			if (!$primary) {
				$sql .= ' AND RDB$INDEX_NAME NOT LIKE \'RDB$%\'';
			} else {
				$sql .= ' AND RDB$INDEX_NAME NOT LIKE \'RDB$FOREIGN%\'';
			}
			// get index details
			$rs = $this->Execute ($sql);
			if (!is_object ($rs) ) {
				// restore fetchmode
				if (isset ($savem) ) {
					$this->SetFetchMode ($savem);
				}

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$indexes = array ();
			while ($row = $rs->FetchRow () ) {
				$index = $row[0];
				if (!isset ($indexes[$index]) ) {
					if (is_null ($row[3]) ) {
						$row[3] = 0;
					}
					$indexes[$index] = array ('unique' => ($row[3] == 1),
								'columns' => array () );
				}
				$sql = 'SELECT * FROM RDB$INDEX_SEGMENTS WHERE RDB$INDEX_NAME = \'' . $index . '\' ORDER BY RDB$FIELD_POSITION ASC';
				$rs1 = $this->Execute ($sql);
				while ($row1 = $rs1->FetchRow () ) {
					$indexes[$index]['columns'][$row1[2]] = $row1[1];
				}
				unset ($row);
			}
			// restore fetchmode
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			return $indexes;

		}

		/**
		* Locks a record.
		*
		* @access public
		* @see http://community.borland.com/article/0,1410,25844,00.html
		* @param string	$tables
		* @param string	$where
		* @param string	$col
		* @return number
		*/

		function RowLock ($tables, $where, $col) {
			if ($this->autoCommit) {
				$this->BeginTrans ();
			}
			$this->Execute ('UPDATE ' . $tables . ' SET ' . $col . '=' . $col . ' WHERE ' . $where . ' ');
			// is this correct?
			return 1;

		}

		/**
		* Prepare the SQL Statement.
		*
		* @access public
		* @param string	$sql
		* @return mixed boolean or array
		*/

		function Prepare ($sql) {

			$stmt = ibase_prepare ($this->_connectionID, $sql);
			if (!$stmt) {
				return false;
			}
			return array ($sql,
					$stmt);

		}

		/**
		* Execute the query.
		*
		* @access private
		* @param string	$sql
		* @param mixed		$iarr
		* @return mixed int or boolean
		*/

		function _query ($sql, $iarr = false) {
			if (!$this->autoCommit && $this->_transactionID) {
				$conn = $this->_transactionID;
				$docommit = false;
			} else {
				$conn = $this->_connectionID;
				$docommit = true;
			}
			if (is_array ($sql) ) {
				$fn = 'ibase_execute';
				$sql = $sql[1];
				if (is_array ($iarr) ) {
					if (version_compare (phpversion (), '4.0.5') == 1) {
						if (!isset ($iarr[0]) ) {
							$iarr[0] = '';
							// PHP5 compat hack
						}
						$fnarr = &array_merge (array ($sql),
										$iarr);
						$ret = call_user_func_array ($fn, $fnarr);
					} else {
						switch (sizeof ($iarr) ) {
							case 1:
								$ret = $fn ($sql, $iarr[0]);
								break;
							case 2:
								$ret = $fn ($sql, $iarr[0], $iarr[1]);
								break;
							case 3:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2]);
								break;
							case 4:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3]);
								break;
							case 5:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4]);
								break;
							case 6:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5]);
								break;
							case 7:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5], $iarr[6]);
								break;
							default:
								$ret = $fn ($sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5], $iarr[6], $iarr[7]);
								opnErrorHandler (E_WARNING, 'Too many parameters to ibase query ' . $sql, __FILE__, __LINE__);
								break;
						}
					}
				} else {
					$ret = $fn ($sql);
				}
			} else {
				$fn = 'ibase_query';
				if (is_array ($iarr) ) {
					if (version_compare (phpversion (), '4.0.5') == 1) {
						if (sizeof ($iarr) == 0) {
							$iarr[0] = '';
							// PHP5 compat hack
						}
						$fnarr = &array_merge (array ($conn,
										$sql),
										$iarr);
						$ret = call_user_func_array ($fn, $fnarr);
					} else {
						switch (sizeof ($iarr) ) {
							case 1:
								$ret = $fn ($conn, $sql, $iarr[0]);
								break;
							case 2:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1]);
								break;
							case 3:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2]);
								break;
							case 4:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3]);
								break;
							case 5:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4]);
								break;
							case 6:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5]);
								break;
							case 7:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5], $iarr[6]);
								break;
							default:
								$ret = $fn ($conn, $sql, $iarr[0], $iarr[1], $iarr[2], $iarr[3], $iarr[4], $iarr[5], $iarr[6], $iarr[7]);
								opnErrorHandler (E_WARNING, 'Too many parameters to ibase query ' . $sql, __FILE__, __LINE__);
								break;
						}
					}
				} else {
					$ret = $fn ($conn, $sql);
				}
			}
			if ($docommit && $ret === true) {
				ibase_commit ($this->_connectionID);
			}
			$this->_thissql = $sql;
			return $ret;

		}

		/**
		* Close the connection.
		*
		* @access private
		* @return number
		*/

		function _close () {
			if (!$this->autoCommit) {
				@ibase_rollback ($this->_connectionID);
			}
			return @ibase_close ($this->_connectionID);

		}

		/**
		* Converts the interbase fieldtypes.
		*
		* @access private
		* @param object	$fld
		* @param int		$ftype
		* @param int		$flen
		* @param int		$fscale
		* @param int		$fsubtype
		* @param int		$fprecision
		* @param bool		$dialect3
		* @return void
		*/

		function _ConvertFieldType (&$fld, $ftype, $flen, $fscale, $fsubtype, $fprecision, $dialect3) {

			$fscale = abs ($fscale);
			$fld->max_length = $flen;
			$fld->scale = null;
			switch ($ftype) {
				case 7:
				case 8:
					if ($dialect3) {
						switch ($fsubtype) {
						case 0:
							$fld->type = ($ftype == 7?'smallint' : 'integer');
							break;
						case 1:
							$fld->type = 'numeric';
							$fld->max_length = $fprecision;
							$fld->scale = $fscale;
							break;
						case 2:
							$fld->type = 'decimal';
							$fld->max_length = $fprecision;
							$fld->scale = $fscale;
							break;
					}
					// switch
				} else {
					if ($fscale != 0) {
						$fld->type = 'decimal';
						$fld->scale = $fscale;
						$fld->max_length = ($ftype == 7?4 : 9);
					} else {
						$fld->type = ($ftype == 7?'smallint' : 'integer');
					}
				}
				break;
				case 16:
					if ($dialect3) {
						switch ($fsubtype) {
						case 0:
							$fld->type = 'decimal';
							$fld->max_length = 18;
							$fld->scale = 0;
							break;
						case 1:
							$fld->type = 'numeric';
							$fld->max_length = $fprecision;
							$fld->scale = $fscale;
							break;
						case 2:
							$fld->type = 'decimal';
							$fld->max_length = $fprecision;
							$fld->scale = $fscale;
							break;
					}
					// switch
				}
				break;
				case 10:
					$fld->type = 'float';
					break;
				case 14:
					$fld->type = 'char';
					break;
				case 27:
					if ($fscale != 0) {
						$fld->type = 'decimal';
						$fld->max_length = 15;
						$fld->scale = 5;
					} else {
						$fld->type = 'double';
					}
					break;
				case 35:
					if ($dialect3) {
						$fld->type = 'timestamp';
					} else {
						$fld->type = 'date';
					}
					break;
				case 12:
					$fld->type = 'date';
					break;
				case 13:
					$fld->type = 'time';
					break;
				case 37:
					$fld->type = 'varchar';
					break;
				case 40:
					$fld->type = 'cstring';
					break;
				case 261:
					$fld->type = 'blob';
					$fld->max_length = -1;
					break;
			}
			// switch

		}

		/**
		* Returns the tablecolumns.
		*
		* @access public
		* @param string	$table
		* @return mixed boolean or array
		*/

		function &MetaColumns ($table) {

			$false = false;
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$rs = $this->Execute (sprintf ($this->metaColumnsSQL, strtoupper ($table) ) );
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			if ($rs === false) {
				return $false;
			}
			$retarr = array ();
			$dialect3 = ($this->dialect == 3?true : false);
			while (! $rs->EOF) {
				// print_r($rs->fields);
				$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
				$fld->name = trim ($rs->fields[0]);
				$this->_ConvertFieldType ($fld, $rs->fields[7], $rs->fields[3], $rs->fields[4], $rs->fields[5], $rs->fields[6], $dialect3);
				if (isset ($rs->fields[1]) && $rs->fields[1]) {
					$fld->not_null = true;
				}
				if (isset ($rs->fields[2]) ) {
					$fld->has_default = true;
					$d = substr ($rs->fields[2], strlen ('default ') );
					switch ($fld->type) {
						case 'smallint':
						case 'integer':
							$fld->default_value = (int) $d;
							break;
						case 'char':
						case 'blob':
						case 'text':
						case 'varchar':
							$fld->default_value = (string)substr ($d, 1, strlen ($d)-2);
							break;
						case 'double':
						case 'float':
							$fld->default_value = (float) $d;
							break;
						default:
							$fld->default_value = $d;
							break;
					}
				}
				if ( (isset ($rs->fields[5]) ) && ($fld->type == 'blob') ) {
					$fld->sub_type = $rs->fields[5];
				} else {
					$fld->sub_type = null;
				}
				if ($this->fetchMode === false) {
					$retarr[] = $fld;
				} else {
					$retarr[strtolower ($fld->name)] = $fld;
				}
				unset ($fld);
				$rs->MoveNext ();
			}
			$rs->Close ();
			if (empty ($retarr) ) {
				return $false;
			}
			return $retarr;

		}

		/**
		* Returns the encoded blob.
		*
		* @access public
		* @param string	$blob
		* @return string
		*/

		function BlobEncode ($blob) {

			$blobid = ibase_blob_create ($this->_connectionID);
			ibase_blob_add ($blobid, $blob);
			return ibase_blob_close ($blobid);

		}

		/**
		* Decodes a blobfield.
		*
		* @access private
		* @param string	$blob
		* @return string
		*/

		function _BlobDecode ($blob) {

			$blobid = ibase_blob_open ($blob);
			$realblob = ibase_blob_get ($blobid, $this->maxblobsize);
			// 2nd param is max size of blob -- Kevin Boillet <kevinboillet@yahoo.fr>
			while ($string = ibase_blob_get ($blobid, 8192) ) {
				$realblob .= $string;
			}
			ibase_blob_close ($blobid);
			return ($realblob);

		}

		/**
		* Updates a blobfield.
		*
		* @access public
		* @param string	$table
		* @param string	$column
		* @param string	$val
		* @param string	$where
		* @param string	$blobtype
		* @return boolean
		*/

		function UpdateBlob ($table, $column, $val, $where, $blobtype = 'BLOB') {

			$t = $blobtype;
			$blob_id = ibase_blob_create ($this->_connectionID);
			// replacement that solves the problem by which only the first modulus 64K /
			// of $val are stored at the blob field ////////////////////////////////////
			// Thx Abel Berenstein  aberenstein#afip.gov.ar
			$len = strlen ($val);
			$chunk_size = 32768;
			$tail_size = $len% $chunk_size;
			$n_chunks = ($len- $tail_size)/ $chunk_size;
			for ($n = 0; $n< $n_chunks; $n++) {
				$start = $n* $chunk_size;
				$data = substr ($val, $start, $chunk_size);
				ibase_blob_add ($blob_id, $data);
			}
			if ($tail_size) {
				$start = $n_chunks* $chunk_size;
				$data = substr ($val, $start, $tail_size);
				ibase_blob_add ($blob_id, $data);
			}
			// end replacement /////////////////////////////////////////////////////////
			$blob_id_str = ibase_blob_close ($blob_id);
			return $this->Execute ('UPDATE ' . $table . ' SET ' . $column . '=(?) WHERE ' . $where, array ($blob_id_str) ) != false;

		}

	}

	/* class SQLLayer_ibase */

	class SQLLayerRecordSet_ibase extends SQLLayerRecordSet {
		var $databaseType = 'ibase';
		var $bind = false;
		var $_cacheType;

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$id
		* @param bool		$mode
		* @return void
		*/

		function SQLLayerRecordSet_ibase ($id, $mode = true) {

			$this->fetchMode = $mode;
			$this->SQLLayerRecordSet ($id);

		}

		/**
		*
		* @param bool	$upper
		* @return array
		*/

		function &GetRowAssoc ($upper = true) {
			if ($this->fetchMode && !$upper) {
				return $this->fields;
			}
			$row = &$this->_GetRowAssoc ($upper);
			return $row;

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return object
		*/

		function &FetchField ($fieldOffset = -1) {

			$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
			$ibf = ibase_field_info ($this->_queryID, $fieldOffset);
			$fld->name = strtolower ($ibf['alias']);
			if (empty ($fld->name) ) {
				$fld->name = strtolower ($ibf['name']);
			}
			$fld->type = $ibf['type'];
			$fld->max_length = $ibf['length'];

			/*	   This needs to be populated from the metadata */

			$fld->not_null = false;
			$fld->has_default = false;
			$fld->default_value = 'null';
			return $fld;

		}

		/**
		* Init the recordset.
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = -1;
			if (is_resource ($this->_queryID) ) {
				$this->_numOfFields = @ibase_num_fields ($this->_queryID);
				// cache types for blob decode check
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$f1 = $this->FetchField ($i);
					if ($this->fetchMode) {
						$this->_cacheType[strtoupper ($f1->name)] = $f1->type;
					} else {
						$this->_cacheType[] = $f1->type;
					}
				}
			}

		}

		/**
		* Fetch a record.
		*
		*
		* @access private
		* @return boolean
		*/

		function _fetch () {
			if (!is_resource ($this->_queryID) ) {
				if (ibase_errmsg () === false) {
					return false;
				}
				return true;
			}
			if ($this->fetchMode) {
				if (version_compare (phpversion (), '4.3.0', 'ge') == 1) {
					$f = ibase_fetch_assoc ($this->_queryID);
				} else {
					$f = get_object_vars (ibase_fetch_object ($this->_queryID) );
				}
			} else {
				$f = @ibase_fetch_row ($this->_queryID);
			}
			if ($f === false) {
				$this->fields = false;
				return false;
			}
			$keys = array_keys ($f);
			foreach ($keys as $i) {
				if ($this->_cacheType[$i] == 'BLOB') {
					if (isset ($f[$i]) ) {
						$f[$i] = $this->connection->_BlobDecode ($f[$i]);
					} else {
						$f[$i] = null;
					}
				} else {
					if (!isset ($f[$i]) ) {
						$f[$i] = null;
					} else
					if (is_string ($f[$i]) ) {
						$f[$i] = rtrim ($f[$i]);
					}
				}
			}
			unset ($keys);
			$this->fields = $f;
			unset ($f);
			if ($this->fetchMode) {
				$this->_convert_to_lowercase ();
			}
			return true;

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return array
		*/

		function Fields ($colname) {
			if ($this->fetchMode) {
				return $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$o = $this->FetchField ($i);
					$this->bind[strtolower ($o->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Close the recordset.
		*
		* @access private
		* @return number
		*/

		function _close () {
			return @ibase_free_result ($this->_queryID);

		}

		/**
		*
		* @access public
		* @param mixed		$t
		* @param int		$len
		* @param object	$fieldobj
		* @return string
		*/

		function MetaType ($t, $len = -1, $fieldobj = object) {
			if (is_object ($t) ) {
				$fieldobj = $t;
				$t = $fieldobj->type;
				$len = $fieldobj->max_length;
			}
			switch (strtoupper ($t) ) {
				case 'CHAR':
					return 'C';
				case 'TEXT':
				case 'VARCHAR':
				case 'VARYING':
					if ($len<=$this->blobSize) {
						return 'C';
					}
					return 'X';
				case 'BLOB':
					return 'B';
				case 'TIMESTAMP':
				case 'DATE':
					return 'D';
				// case 'T':
				case 'TIME':
					return 'T';
				// case 'L':
				// return 'L';
				case 'INT':
				case 'SHORT':
				case 'INTEGER':
					return 'I';
				default:
					return 'N';
			}

		}

	}

	/* class SQLLayerRecordSet_ibase */

	class opn_ibase extends OPN_SQL {
		var $DropTable = 'DROP TABLE %s';
		var $DropIndex = 'DROP INDEX %s';

		function opn_ibase () {

			$this->CanOptimize = false;
			$this->CanRepair = false;
			$this->HasStatus = false;
			$this->CanAlterColumnAdd = false;
			$this->CanAlterColumnChange = false;
			$this->CanAddRepairField = true;
			$this->CanChangeRepairField = true;
			$this->CanExecuteColumnAdd = false;
			$this->CanExecuteColumnChange = false;

		}

		function TableComment ($tablename, $comment) {
			return "UPDATE rdb\$relations SET rdb\$description='" . $comment . "' WHERE  rdb\$relation_name=upper('$tablename')";

		}

		function TableDrop ($tablename) {
			return 'DROP TABLE ' . $tablename;

		}

		/* OPN_SQL::ChangeTablename()
		*
		* @param  $module
		* @param  $oldname
		* @param  $newname
		* @return
		*/

		function ChangeTablename ($module, $oldname, $newname) {

			global $opnConfig, $opnTables;
			if ($this->_CreateTable ($module, $newname, $newname) ) {
				$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . $newname . ' SELECT * FROM ' . $opnConfig['tableprefix'] . $oldname;
				$opnConfig['database']->StartTrans ();
				$opnConfig['database']->Execute ($insertquery);
				$opnConfig['database']->CompleteTrans ();
				$drop = $this->TableDrop ($opnConfig['tableprefix'] . $oldname);
				$opnConfig['database']->Execute ($drop);
				$opnConfig['database']->Execute ('UPDATE ' . $opnConfig['tableprefix'] . 'dbcat' . " SET value1='" . $newname . "' WHERE value1='" . $oldname . "'");
				dbconf_get_tables ($opnTables, $opnConfig);
			}

		}

		function TableRename ($oldname, $newname) {
			return "UPDATE RDB\$RELATIONS SET RDB\$RELATION_NAME='" . strtoupper ($newname) . "' WHERE RDB\$RELATION_NAME='" . str_pad (strtoupper ($oldname), 31) . "'";

		}

		function FieldRename ($module, $table, $oldname, $newname) {

			global $opnConfig;

			$t = $module;
			$opnConfig['database']->StartTrans ();
			$opnConfig['database']->Execute ('ALTER TABLE ' . strtoupper ($opnConfig['tableprefix'] . $table) . ' ALTER COLUMN ' . strtoupper ($oldname) . ' TO ' . strtoupper ($newname) );
			$opnConfig['database']->CompleteTrans ();

		}

		function IndexDrop ($index, $table = '') {

			$t = $table;
			return 'DROP INDEX ' . $index;

		}

		function PrimaryIndexDrop ($table) {

			global $opnConfig;

			$result = $opnConfig['database']->Execute ("SELECT RDB\$CONSTRAINT_NAMEFROMRDB\$RELATION_CONSTRAINTSWHERE(RDB\$CONSTRAINTTYPE='PRIMARYKEY')AND(RDB\$RELATION_NAME=' $table')");
			$constraint = $result->fields['RDB$CONSTRAINT_NAME'];
			$result->Close ();
			unset ($result);
			return 'ALTER TABLE ' . $table . ' DROP CONSTRAINT ' . $constraint;

		}

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {
			return sprintf ($this->IndexCreate, $unique, $idxname, $idxcounter, $table, $fields);

		}

		function CreateOpnRegexp ($field, $regx) {
			if ($regx == '0-9') {
				$where = "( ";
				$where .= "(" . $field . " like '1%') OR ";
				$where .= "(" . $field . " like '2%') OR ";
				$where .= "(" . $field . " like '3%') OR ";
				$where .= "(" . $field . " like '4%') OR ";
				$where .= "(" . $field . " like '5%') OR ";
				$where .= "(" . $field . " like '6%') OR ";
				$where .= "(" . $field . " like '7%') OR ";
				$where .= "(" . $field . " like '8%') OR ";
				$where .= "(" . $field . " like '9%') OR ";
				$where .= "(" . $field . " like '0%')";
				$where .= " )";
			} elseif ($regx == 'a-z') {
				$where = "( ";
				$where .= "(" . $field . " like 'a%') OR ";
				$where .= "(" . $field . " like 'b%') OR ";
				$where .= "(" . $field . " like 'c%') OR ";
				$where .= "(" . $field . " like 'd%') OR ";
				$where .= "(" . $field . " like 'e%') OR ";
				$where .= "(" . $field . " like 'f%') OR ";
				$where .= "(" . $field . " like 'g%') OR ";
				$where .= "(" . $field . " like 'h%') OR ";
				$where .= "(" . $field . " like 'i%') OR ";
				$where .= "(" . $field . " like 'j%') OR ";
				$where .= "(" . $field . " like 'k%') OR ";
				$where .= "(" . $field . " like 'l%') OR ";
				$where .= "(" . $field . " like 'm%') OR ";
				$where .= "(" . $field . " like 'n%') OR ";
				$where .= "(" . $field . " like 'o%') OR ";
				$where .= "(" . $field . " like 'p%') OR ";
				$where .= "(" . $field . " like 'q%') OR ";
				$where .= "(" . $field . " like 'r%') OR ";
				$where .= "(" . $field . " like 's%') OR ";
				$where .= "(" . $field . " like 't%') OR ";
				$where .= "(" . $field . " like 'u%') OR ";
				$where .= "(" . $field . " like 'v%') OR ";
				$where .= "(" . $field . " like 'w%') OR ";
				$where .= "(" . $field . " like 'x%') OR ";
				$where .= "(" . $field . " like 'y%') OR ";
				$where .= "(" . $field . " like 'z%')";
				$where .= " )";
			}
			return $where;

		}

		function GetIndex ($table) {
			return "SELECTrdb\$index_name,rdb\$relation_nameFROMrdb\$indicesWHERErdb\$relation_name=upper(' $table')ANDnot(rdb\$index_namelike'RDB%')";

		}

		function ChangeColumn ($module, $table, $field) {

			global $opnConfig;

			$t = $field;
			if ($this->_CreateTable ($module, 'dummy', $table) ) {
				$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . 'dummy SELECT * FROM ' . $opnConfig['tableprefix'] . $table;
				$opnConfig['database']->StartTrans ();
				$opnConfig['database']->Execute ($insertquery);
				$opnConfig['database']->CompleteTrans ();
				$drop = $this->TableDrop ($opnConfig['tableprefix'] . $table);
				$opnConfig['database']->Execute ($drop);
				$this->_CreateTable ($module, $table, $table);
				$dbfields = $opnConfig['database']->MetaColumns ($opnConfig['tableprefix'] . 'dummy');
				$dbfields = array_keys ($dbfields);
				$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . $table . ' SELECT ' . implode (',', $dbfields) . ' FROM ' . $opnConfig['tableprefix'] . 'dummy';
				unset ($dbfields);
				$opnConfig['database']->StartTrans ();
				$opnConfig['database']->Execute ($insertquery);
				$opnConfig['database']->CompleteTrans ();
				$drop = $this->TableDrop ($opnConfig['tableprefix'] . 'dummy');
				$opnConfig['database']->Execute ($drop);
			}

		}

		function AddRepairField ($module, $table, $field) {

			global $opnConfig;

			$t = $field;
			if ($this->_CreateTable ($module, 'dummy', $table, true) ) {
				$module1 = explode ('/', $module);
				$module1 = $module1[1];
				$thefunc = $module1 . '_repair_sql_table';
				if (function_exists ($thefunc) ) {
					$arr = $thefunc ();
					$insertquery = '';
					$result = &$opnConfig['database']->Execute ('SELECT * FROM ' . $opnConfig['tableprefix'] . $table);
					if ($result !== false) {
						while (! $result->EOF) {
							$row = $result->GetRowAssoc ('0');
							$first = 0;
							$insertquery .= 'INSERT INTO "' . $opnConfig['tableprefix'] . 'dummy" (';
							$values = ') VALUES (';
							$tmparr = array_keys ($arr['table'][$table]);
							foreach ($tmparr as $fieldname) {
								if (strpos ($fieldname, '__opn_key') ) {
								} else {
									if ($first != 0) {
										$values .= ', ';
										$insertquery .= ', ';
									}
									if (isset ($row[$fieldname]) ) {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "'" . addslashes ($row[$fieldname]) . "'";
									} else {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "''";
									}
									$first++;
								}
							}
							$insertquery .= $values . ');';
							$result->MoveNext ();
						}
						$drop = $this->TableDrop ($opnConfig['tableprefix'] . $table);
						$opnConfig['database']->Execute ($drop);
					}
					$this->_CreateTable ($module, $table, $table, true);
					if ($insertquery != '') {
						$opnConfig['database']->Execute ($insertquery);
						$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . $table . ' SELECT * FROM ' . $opnConfig['tableprefix'] . 'dummy';
						$opnConfig['database']->Execute ($insertquery);
					}
					$drop = $this->TableDrop ($opnConfig['tableprefix'] . 'dummy');
					$opnConfig['database']->Execute ($drop);
				}
			}

		}

		function ChangeRepairField ($module, $table, $field) {

			$this->ChangeColumn ($module, $table, $field);

		}

		function GetDBType ($datatype, $length = 0, $default = null, $null = false, $decimal = 0) {

			global $opnConfig;
			if ( (isset ($opnConfig['return_tableinfo_as_array']) ) && ($opnConfig['return_tableinfo_as_array'] === TRUE) ) {
				if ($decimal == 0) {
					$decimal = null;
				}
				return $opnConfig['opnSQL']->BuildField ('', $length, $datatype, ! $null, !is_null ($default), $default, $decimal);
			}
			$hlp = '';
			switch ($datatype) {
				case _OPNSQL_TINYINT:
				case _OPNSQL_SMALLINT:
					$hlp .= 'SMALLINT';
					break;
				case _OPNSQL_MEDIUMINT:
				case _OPNSQL_INT:
				case _OPNSQL_BIGINT:
					$hlp .= 'INTEGER';
					break;
				case _OPNSQL_FLOAT:
					$hlp .= 'FLOAT';
					break;
				case _OPNSQL_DOUBLE:
					$hlp .= 'DOUBLE PRECISION';
					break;
				case _OPNSQL_NUMERIC:
					$hlp .= 'DECIMAL (' . $length . ',' . $decimal . ')';
					break;
				case _OPNSQL_DATE:
				case _OPNSQL_DATETIME:
				case _OPNSQL_TIMESTAMP:
				case _OPNSQL_TIME:
					$hlp .= 'DATE';
					break;
				case _OPNSQL_CHAR:
					$hlp .= 'CHAR (' . $length . ')';
					break;
				case _OPNSQL_VARCHAR:
					$hlp .= 'VARCHAR (' . $length . ')';
					break;
				case _OPNSQL_TINYBLOB:
				case _OPNSQL_BLOB:
				case _OPNSQL_MEDIUMBLOB:
				case _OPNSQL_BIGBLOB:
					$hlp .= 'BLOB SUB_TYPE 0 SEGMENT SIZE 4096';
					break;
				case _OPNSQL_TINYTEXT:
				case _OPNSQL_TEXT:
				case _OPNSQL_MEDIUMTEXT:
				case _OPNSQL_BIGTEXT:
					$hlp .= 'BLOB SUB_TYPE 1 SEGMENT SIZE 4096';
					break;
			}
			if ($default !== NULL) {
				$hlp .= ' DEFAULT ';
				if (is_string ($default) ) {
					if (strlen ($default) == 0) {
						$hlp .= "''";
					} else {
						$hlp .= "'" . $default . "'";
					}
				} else {
					$hlp .= $default;
				}
			}
			if ($null) {
			} else {
				$hlp .= ' NOT NULL';
			}
			return $hlp;

		}

		function GetDbFieldtype (&$modulefield) {
			switch ($modulefield->type) {
				case _OPNSQL_TINYINT:
				case _OPNSQL_SMALLINT:
					$modulefield->type = 'smallint';
					break;
				case _OPNSQL_MEDIUMINT:
				case _OPNSQL_INT:
				case _OPNSQL_BIGINT:
					$modulefield->type = 'integer';
					break;
				case _OPNSQL_FLOAT:
					$modulefield->type = 'float';
					break;
				case _OPNSQL_DOUBLE:
					$modulefield->type = 'double';
					break;
				case _OPNSQL_NUMERIC:
					$modulefield->type = 'decimal';
					break;
				case _OPNSQL_DATE:
				case _OPNSQL_DATETIME:
				case _OPNSQL_TIMESTAMP:
				case _OPNSQL_TIME:
					$modulefield->type = 'date';
					break;
				case _OPNSQL_CHAR:
					$modulefield->type = 'char';
					break;
				case _OPNSQL_VARCHAR:
					$modulefield->type = 'varchar';
					break;
				case _OPNSQL_TINYBLOB:
				case _OPNSQL_BLOB:
				case _OPNSQL_MEDIUMBLOB:
				case _OPNSQL_BIGBLOB:
					$modulefield->type = 'blob subtype 0';
					break;
				case _OPNSQL_TINYTEXT:
				case _OPNSQL_TEXT:
				case _OPNSQL_MEDIUMTEXT:
				case _OPNSQL_BIGTEXT:
					$modulefield->type = 'blob subtype 1';
					break;
			}
			// switch

		}

		function CheckDBField ($dbfield, &$modulefield) {

			$this->GetDbFieldtype ($modulefield);
			if ($dbfield->type == 'blob') {
				$dbfield->type .= ' subtype ' . $dbfield->sub_type;
			}
			$help = '';
			if (strtolower ($dbfield->name) != strtolower ($modulefield->name) ) {
				$help[] = _OPNSQL_FIELDERRORNAME;
			}
			if ( ($modulefield->type == 'char') || ($modulefield->type == 'varchar') || ($modulefield->type == 'decimal') || ($modulefield->type == 'numeric') ) {
				if ( ($dbfield->max_length != -1) && ($dbfield->max_length != $modulefield->max_length) ) {
					$help[] = _OPNSQL_FIELDERRORLENGTH;
				}
			}
			if ($dbfield->type != $modulefield->type) {
				$help[] = _OPNSQL_FIELDERRORTYPE;
			}
			if ($dbfield->not_null != $modulefield->not_null) {
				$help[] = _OPNSQL_FIELDERRORNOTNULL;
			}
			if ( ($modulefield->has_default) && ($dbfield->has_default != $modulefield->has_default) ) {
				$help[] = _OPNSQL_FIELDERRORHASDEFAULT;
			}
			if ( (!is_null ($dbfield->scale) ) && ($dbfield->scale != $modulefield->scale) ) {
				$help[] = _OPNSQL_FIELDERRORSCALE;
			}
			return $help;

		}

		/**
		*
		* @param $s
		* @param $magic_quotes
		* @return string
		*/

		function qstr ($s, $magic_quotes = false) {

			global $opnConfig;
			if ( ($magic_quotes !== false) && ($magic_quotes !== true) ) {
				if ($this->_blobcache === false) {
					$this->_blobcache = array ();
				}
				// end if
				$this->_blobcache[$magic_quotes] = $s;
				return "''";
			}
			// end if
			return $opnConfig['database']->qstr ($s, $magic_quotes);

		}
		// end function

		function UpdateBlobs ($table, $where, $blobtype = 'BLOB') {

			global $opnConfig;
			if ( ($this->_blobcache !== false) && (is_array ($this->_blobcache) ) ) {
				foreach ($this->_blobcache as $key => $vals) {
					$opnConfig['database']->UpdateBlob ($table, $key, $vals, $where, $blobtype);
				}
				// end foreach
				$this->_blobcache = false;
				return true;
			}
			// end if
			return false;

		}
		// end function

	}
}

?>