<?php
/*
* OpenPHPNuke: Great Web Portal System
*
* Copyright (c) 2001-2006 by
* Heinz Hombergs heinz@hhombergs.de
* Stefan Kaletta stefan@kaletta.de
* Alexander Weber xweber@kamelfreunde.de
*
* This program is free software. You can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License.
* See LICENSE for details.
*
* This software is based on various code found in the internet
* See CREDITS for details
* If you are an author of a part of opn and not you are not mentioned
* SORRY for that ! We respect your rights to your code, so please send
* an eMail to devteam@openphpnuke.com we will include you to the CREDITS asap
*/
if (!defined ('_OPN_CLASS_DRIVERSQLITE_INCLUDED') ) {
	define ('_OPN_CLASS_DRIVERSQLITE_INCLUDED', 1);

	/**
	* Extends the SQL Connection class for SQLITE Database access
	*
	* @var string	$dataProvider
	* @var string	$databaseType
	* @var string	$replaceQuote	- string to use to replace quotes
	* @var integer	$_errorNo
	* @var string	$metaTablesSQL
	*/

	class SQLLayer_sqlite extends SQLConnection {

		/*<{php4}>
		<{php4/}>*/

		/*<{php5}>
		<{php5/}>*/

		/*<{delete}>*/
		/*<{delete/}>*/
		var $dataProvider = 'sqlite';
		var $databaseType = 'sqlite';
		var $replaceQuote = "''";
		// string to use to replace quotes
		var $_errorNo = 0;
		var $metaTablesSQL = 'SELECT name FROM sqlite_master WHERE type=\'table\' ORDER BY name';

		/**
		* Constructor for SQLLayer_sqlite
		*
		* @access public
		* @return void
		*/

		function SQLLayer_sqlite () {

			$this->fetchMode = true;

		}

		/**
		* Start transaction.
		*
		* @access public
		* @return boolean
		*/

		function BeginTrans () {
			if (!$this->transOff) {
				$this->Execute ('BEGIN TRANSACTION');
				$this->transCnt += 1;
			}
			return true;

		}

		/**
		* Commit transaction.
		*
		* @access public
		* @param bool	$ok
		* @return boolean
		*/

		function CommitTrans ($ok = true) {
			if ($this->transOff) {
				return true;
			}
			if (!$ok) {
				return $this->RollbackTrans ();
			}
			$ret = $this->Execute ('COMMIT');
			if ($this->transCnt>0) {
				$this->transCnt -= 1;
			}
			return !empty ($ret);

		}

		/**
		* Rollback transaction.
		*
		* @access public
		* @return boolean
		*/

		function RollbackTrans () {
			if ($this->transOff) {
				return true;
			}
			$ret = $this->Execute ('ROLLBACK');
			if ($this->transCnt>0) {
				$this->transCnt -= 1;
			}
			return !empty ($ret);

		}

		/**
		* Return the errormessage.
		*
		* @access public
		* @return string
		*/

		function ErrorMsg () {

			$this->_errorMsg = ($this->_errorNo)?sqlite_error_string ($this->_errorNo) : '';
			return $this->_errorMsg;

		}

		/**
		* Returns the errornumber.
		*
		* @access public
		* @return int
		*/

		function ErrorNo () {

			$this->_errorCode = $this->_errorNo;
			return $this->_errorCode;

		}

		/**
		* List columns in a database as an array
		*
		* @access public
		* @param string	$tab
		* @return mixed unknown or array
		*/

		function &MetaColumns ($tab) {

			$rs = $this->Execute ('SELECT * FROM ' . $tab . ' LIMIT 1');
			if (!$rs) {

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$arr = array ();
			for ($i = 0, $max = $rs->_numOfFields; $i< $max; $i++) {
				$fld = &$rs->FetchField ($i);
				if (!$this->fetchMode) {
					$arr[] = &$fld;
				} else {
					$arr[strtoupper ($fld->name)] = &$fld;
				}
			}
			$rs->Close ();
			return $arr;

		}

		/**
		* Connect to a database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return mixed unknown or boolean
		*/

		function _connect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$t = $argHostname;
			$t = $argUsername;
			$t = $argPassword;
			$this->_connectionID = sqlite_open ($argDatabasename);
			if ($this->_connectionID === false) {
				return false;
			}
			return true;

		}

		/**
		* Persistent connection to a database.
		*
		* @access private
		* @param string	$argHostname
		* @param string	$argUsername
		* @param string	$argPassword
		* @param string	$argDatabasename
		* @return mixed unknown or boolean
		*/

		function _pconnect ($argHostname, $argUsername, $argPassword, $argDatabasename) {

			$t = $argHostname;
			$t = $argUsername;
			$t = $argPassword;
			$this->_connectionID = sqlite_popen ($argDatabasename);
			if ($this->_connectionID === false) {
				return false;
			}
			return true;

		}

		/**
		* Excute the query
		*
		* Returns the queryID or the errornumber
		*
		* @access private
		* @param string	$sql
		* @param mixed		$inputarr
		* @return resource
		*/

		function _query ($sql, $inputarr = false) {

			$t = $inputarr;
			$rez = sqlite_query ($sql, $this->_connectionID);
			if (!$rez) {
				$this->_errorNo = sqlite_last_error ($this->_connectionID);
			}
			return $rez;

		}

		/**
		* select records, start getting rows from $offset (1-based), for $nrows.
		*
		* @access public
		* @param string	$sql
		* @param int		$nrows
		* @param int		$offset
		* @param mixed		$inputarr
		* @return void
		*/

		function &SelectLimit ($sql, $nrows = -1, $offset = -1, $inputarr = false) {

			$offsetStr = ($offset >= 0)?' OFFSET ' . $offset : '';
			$limitStr = ($nrows >= 0)?' LIMIT ' . $nrows : ($offset >= 0?' LIMIT 999999999' : '');
			$rs = &$this->Execute ($sql . $limitStr . $offsetStr, $inputarr);
			return $rs;

		}

		/**
		* Close the database connection.
		*
		* @access private
		* @return number
		*/

		function _close () {
			return @sqlite_close ($this->_connectionID);

		}

		/**
		* List indexes on a table as an array
		*
		* @access public
		* @param string	$table
		* @param bool		$primary
		* @param mixed		$owner
		* @return mixed boolean or array
		*/

		function &MetaIndexes ($table, $primary = false, $owner = false) {

			$t = $owner;
			// save old fetch mode
			if ($this->fetchMode !== false) {
				$savem = $this->SetFetchMode (false);
			}
			$sql = sprintf ('SELECT name, sql FROM sqlite_master WHERE type=\'index\' AND tbl_name=\'%s\'', strtolower ($table) );
			$rs = $this->Execute ($sql);
			if (!is_object ($rs) ) {
				if (isset ($savem) ) {
					$this->SetFetchMode ($savem);
				}

				/* return byRef required - so it must be a variable*/

				$false = false;
				return $false;
			}
			$indexes = array ();
			while ($row = $rs->FetchRow () ) {
				if ($primary && preg_match ("/primary/i", $row[1]) == 0) {
					continue;
				}
				if (!isset ($indexes[$row[0]]) ) {
					$indexes[$row[0]] = array ('unique' => preg_match ("/unique/i",
								$row[1]),
								'columns' => array () );
				}

				/**
				* There must be a more elegant way of doing this,
				* the index elements appear in the SQL statement
				* in cols[1] between parentheses
				* e.g CREATE UNIQUE INDEX ware_0 ON warehouse (org,warehouse)
				*/

				$cols = explode ('(', $row[1]);
				$cols = explode (')', $cols[1]);
				array_pop ($cols);
				$indexes[$row[0]]['columns'] = $cols;
			}
			if (isset ($savem) ) {
				$this->SetFetchMode ($savem);
			}
			return $indexes;

		}

	}

	/* class SQLLayer_sqlite */

	/**
	* Extends the RecordSet class that represents the dataset returned by the database
	* for sqLite
	*
	* @var string	$databaseType
	* @var boolean	$bind
	* @var boolean	$canSeek
	*/

	class SQLLayerRecordSet_sqlite extends SQLLayerRecordSet {
		var $databaseType = 'sqlite';
		var $bind = false;
		var $canSeek = true;

		/**
		* Constructor
		*
		* @access public
		* @param ressource	$id
		* @param bool		$mode
		* @return void
		*/

		function SQLLayerRecordSet_sqlite ($id, $mode = true) {
			if ($mode) {
				$this->fetchMode = SQLITE_ASSOC;
			} else {
				$this->fetchMode = SQLITE_NUM;
			}
			$this->_queryID = $id;
			$this->_inited = true;
			$this->fields = array ();
			if ($id) {
				$this->_currentRow = 0;
				$this->EOF = ! $this->_fetch ();
				@ $this->_initrs ();
			} else {
				$this->_numOfRows = 0;
				$this->_numOfFields = 0;
				$this->EOF = true;
			}
			return $this->_queryID;

		}

		/**
		* Get the SQLLayerFieldObject of a specific column.
		*
		* @access public
		* @param int	$fieldOffset
		* @return object
		*/

		function &FetchField ($fieldOffset = -1) {

			$fld =  /*<{4php5}>& new!!new<{4php5/}>*//*<{delete}>*/& new/*<{delete/}>*/ SQLLayerFieldObject;
			$fld->name = sqlite_field_name ($this->_queryID, $fieldOffset);
			$fld->type = 'VARCHAR';
			$fld->max_length = -1;
			return $fld;

		}

		/**
		* Initialize recordset
		*
		* @access private
		* @return void
		*/

		function _initrs () {

			$this->_numOfRows = @sqlite_num_rows ($this->_queryID);
			$this->_numOfFields = @sqlite_num_fields ($this->_queryID);

		}

		/**
		* Get the value of a field in the current row by column name.
		*
		* @access public
		* @param string	$colname
		* @return array
		*/

		function Fields ($colname) {
			if ($this->fetchMode != SQLITE_NUM) {
				return $this->fields[$colname];
			}
			if (!$this->bind) {
				$this->bind = array ();
				for ($i = 0, $max = $this->_numOfFields; $i< $max; $i++) {
					$o = $this->FetchField ($i);
					$this->bind[strtolower ($o->name)] = $i;
				}
			}
			return $this->fields[$this->bind[strtolower ($colname)]];

		}

		/**
		* Move to the first record.
		*
		* @access public
		* @return boolean
		*/

		function MoveFirst () {
			if ($this->_numOfRows) {
				sqlite_rewind ($this->_queryID);
			}
			if ($this->_fetch () ) {
				return true;
			}
			return false;

		}

		/**
		* Adjusts the result pointer to an arbitary row in the result
		*
		* @access private
		* @param int	$row
		* @return bool
		*/

		function _seek ($row) {
			return sqlite_seek ($this->_queryID, ($row+1) );

		}

		/**
		* set $fields to row values
		*
		* @access private
		* @param bool	$ignore_fields
		* @return boolean
		*/

		function _fetch ($ignore_fields = false) {

			$t = $ignore_fields;
			$this->fields = @sqlite_fetch_array ($this->_queryID, $this->fetchMode);
			if ( ($this->fetchMode == SQLITE_ASSOC) && (is_array ($this->fields) ) ) {
				$this->_convert_to_lowercase ();
			}
			return !empty ($this->fields);

		}

		/**
		* Close the recordset
		*
		* @access private
		* @return void
		*/

		function _close () {

		}

	}

	/* class SQLLayerRecordSet_sqlite */

	class opn_sqlite extends OPN_SQL {

		function opn_sqlite () {

			$this->CanOptimize = false;
			$this->CanRepair = false;
			$this->HasStatus = false;
			$this->CanAlterColumnAdd = false;
			$this->CanAlterColumnChange = false;
			$this->CanAddRepairField = true;
			$this->CanChangeRepairField = false;
			$this->CanExecuteColumnAdd = false;
			$this->CanExecuteColumnChange = false;

		}

		function TableComment ($tablename, $comment) {
			return 'COMMENT ON ' . $tablename . " IS '" . $comment . "'";

		}

		function TableOptimize ($tablename) {

			$t = $tablename;
			return '';

		}

		function TableRepair ($tablename) {

			$t = $tablename;
			return '';

		}

		function AddRepairField ($module, $table, $field) {

			global $opnConfig;

			$t = $field;
			if ($this->_CreateTable ($module, 'dummy', $table, true) ) {
				$module1 = explode ('/', $module);
				$module1 = $module1[1];
				$thefunc = $module1 . '_repair_sql_table';
				if (function_exists ($thefunc) ) {
					$arr = $thefunc ();
					$insertquery = '';
					$result = &$opnConfig['database']->Execute ('SELECT * FROM ' . $opnConfig['tableprefix'] . $table);
					if ($result !== false) {
						while (! $result->EOF) {
							$row = $result->GetRowAssoc ('0');
							$first = 0;
							$insertquery .= 'INSERT INTO "' . $opnConfig['tableprefix'] . 'dummy" (';
							$values = ') VALUES (';
							$tmparr = array_keys ($arr['table'][$table]);
							foreach ($tmparr as $fieldname) {
								if (strpos ($fieldname, '__opn_key') ) {
								} else {
									if ($first != 0) {
										$values .= ', ';
										$insertquery .= ', ';
									}
									if (isset ($row[$fieldname]) ) {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "'" . addslashes ($row[$fieldname]) . "'";
									} else {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "''";
									}
									$first++;
								}
							}
							$insertquery .= $values . ');';
							$result->MoveNext ();
						}
						$drop = $this->TableDrop ($opnConfig['tableprefix'] . $table);
						$opnConfig['database']->Execute ($drop);
					}
					$this->_CreateTable ($module, $table, $table, true);
					if ($insertquery != '') {
						$opnConfig['database']->Execute ($insertquery);
						$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . $table . ' SELECT * FROM ' . $opnConfig['tableprefix'] . 'dummy';
						$opnConfig['database']->Execute ($insertquery);
					}
					$drop = $this->TableDrop ($opnConfig['tableprefix'] . 'dummy');
					$opnConfig['database']->Execute ($drop);
				}
			}

		}

		function ChangeTablename ($module, $oldname, $newname) {

			global $opnConfig, $opnTables;
			if ($this->_CreateTable ($module, $newname, $newname, true) ) {
				$module1 = explode ('/', $module);
				$module1 = $module1[1];
				$thefunc = $module1 . '_repair_sql_table';
				if (function_exists ($thefunc) ) {
					$arr = $thefunc ();
					$insertquery = '';
					$result = &$opnConfig['database']->Execute ('SELECT * FROM ' . $opnConfig['tableprefix'] . $newname);
					if ($result !== false) {
						while (! $result->EOF) {
							$row = $result->GetRowAssoc ('0');
							$first = 0;
							$insertquery .= 'INSERT INTO "' . $opnConfig['tableprefix'] . $newname . '" (';
							$values = ') VALUES (';
							$tmparr = array_keys ($arr['table'][$newname]);
							foreach ($tmparr as $fieldname) {
								if (strpos ($fieldname, '__opn_key') ) {
								} else {
									if ($first != 0) {
										$values .= ', ';
										$insertquery .= ', ';
									}
									if (isset ($row[$fieldname]) ) {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "'" . addslashes ($row[$fieldname]) . "'";
									} else {
										$insertquery .= '"' . $fieldname . '"';
										$values .= "''";
									}
									$first++;
								}
							}
							$insertquery .= $values . ');';
							$result->MoveNext ();
						}
						$drop = $this->TableDrop ($opnConfig['tableprefix'] . $oldname);
						$opnConfig['database']->Execute ($drop);
					}
					if ($insertquery != '') {
						$opnConfig['database']->Execute ($insertquery);
					}
				}
			}
			$opnConfig['database']->Execute ('UPDATE ' . $opnConfig['tableprefix'] . 'dbcat' . " SET value1='" . $newname . "' WHERE value1='" . $oldname . "'");
			dbconf_get_tables ($opnTables, $opnConfig);

		}

		function ChangeFieldname ($module, $table, $oldname, $newname, $datatype = '', $length = 0, $default = NULL, $null = FALSE, $decimal = 0) {

			global $opnConfig;

			$t = $datatype;
			$t = $length;
			$t = $default;
			$t = $null;
			$t = $decimal;
			if ($this->_CreateTable ($module, 'dummy', $table, true) ) {
				$module1 = explode ('/', $module);
				$module1 = $module1[1];
				$thefunc = $module1 . '_repair_sql_table';
				if (function_exists ($thefunc) ) {
					$arr = $thefunc ();
					$olddbfields = $opnConfig['database']->MetaColumns ($opnConfig['tableprefix'] . $table);
					$oldnames = '';
					$newnames = '';
					foreach ($olddbfields as $key => $item) {
						$oldnames .= $item->name . ', ';
						if ($key != strtoupper ($oldname) ) {
							$newnames .= $item->name . ', ';
						} else {
							$newnames .= $newname . ', ';
						}
					}
					$oldnames = substr ($oldnames, 0, -2);
					$newnames = substr ($newnames, 0, -2);
					$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . 'dummy (' . $newnames . ') SELECT ' . $oldnames . ' FROM ' . $opnConfig['tableprefix'] . $table;
					$opnConfig['database']->Execute ($insertquery);
					$drop = $this->TableDrop ($opnConfig['tableprefix'] . $table);
					$opnConfig['database']->Execute ($drop);
					$this->_CreateTable ($module, $table, $table, true);
					$insertquery = 'INSERT INTO ' . $opnConfig['tableprefix'] . $table . ' SELECT * FROM ' . $opnConfig['tableprefix'] . 'dummy';
					$opnConfig['database']->Execute ($insertquery);
					$drop = $this->TableDrop ($opnConfig['tableprefix'] . 'dummy');
					$opnConfig['database']->Execute ($drop);
				}
			}

		}

		/**
		* Rename a field
		*
		* @param string	$module
		* @param string	$table
		* @param string	$oldname
		* @param string	$newname
		* @return void
		*/

		function FieldRename ($module, $table, $oldname, $newname) {

			$t = $newname;
			$this->AddRepairField ($module, $table, $oldname);

		}

		function ChangeColumn ($module, $table, $field) {

			$this->AddRepairField ($module, $table, $field);

		}

		function TableDrop ($tablename) {
			return 'DROP TABLE ' . $tablename;

		}

		function TableRename ($oldname, $newname) {
			return 'ALTER TABLE ' . $oldname . ' RENAME TO ' . $newname;

		}

		function CreateIndex ($unique, $idxname, $idxcounter, $table, $fields) {
			return sprintf ($this->IndexCreate, $unique, $idxname, $idxcounter, $table, $fields);

		}

		function IndexDrop ($index, $table = '') {

			$t = $table;
			return 'DROP INDEX "' . $index . '"';

		}

		/* To Do */

		function PrimaryIndexDrop ($table) {

			$t = $table;
			return '';

		}

		function CreateOpnRegexp ($field, $regx) {
			if ($regx == '0-9') {
				$where = "( ";
				$where .= "(" . $field . " like '1%') OR ";
				$where .= "(" . $field . " like '2%') OR ";
				$where .= "(" . $field . " like '3%') OR ";
				$where .= "(" . $field . " like '4%') OR ";
				$where .= "(" . $field . " like '5%') OR ";
				$where .= "(" . $field . " like '6%') OR ";
				$where .= "(" . $field . " like '7%') OR ";
				$where .= "(" . $field . " like '8%') OR ";
				$where .= "(" . $field . " like '9%') OR ";
				$where .= "(" . $field . " like '0%')";
				$where .= " )";
			} elseif ($regx == 'a-z') {
				$where = "( ";
				$where .= "(" . $field . " like 'a%') OR ";
				$where .= "(" . $field . " like 'b%') OR ";
				$where .= "(" . $field . " like 'c%') OR ";
				$where .= "(" . $field . " like 'd%') OR ";
				$where .= "(" . $field . " like 'e%') OR ";
				$where .= "(" . $field . " like 'f%') OR ";
				$where .= "(" . $field . " like 'g%') OR ";
				$where .= "(" . $field . " like 'h%') OR ";
				$where .= "(" . $field . " like 'i%') OR ";
				$where .= "(" . $field . " like 'j%') OR ";
				$where .= "(" . $field . " like 'k%') OR ";
				$where .= "(" . $field . " like 'l%') OR ";
				$where .= "(" . $field . " like 'm%') OR ";
				$where .= "(" . $field . " like 'n%') OR ";
				$where .= "(" . $field . " like 'o%') OR ";
				$where .= "(" . $field . " like 'p%') OR ";
				$where .= "(" . $field . " like 'q%') OR ";
				$where .= "(" . $field . " like 'r%') OR ";
				$where .= "(" . $field . " like 's%') OR ";
				$where .= "(" . $field . " like 't%') OR ";
				$where .= "(" . $field . " like 'u%') OR ";
				$where .= "(" . $field . " like 'v%') OR ";
				$where .= "(" . $field . " like 'w%') OR ";
				$where .= "(" . $field . " like 'x%') OR ";
				$where .= "(" . $field . " like 'y%') OR ";
				$where .= "(" . $field . " like 'z%')";
				$where .= " )";
			}
			return $where;

		}

		function GetIndex ($table) {
			return 'SELECT ic.relname AS index_name, bc.relname AS tab_name, i.indisunique AS unique_key, i.indisprimary AS primary_key FROM pg_class bc, pg_class ic, pg_index i, pg_attribute ia WHERE bc.oid = i.indrelid AND ic.oid = i.indexrelid AND ia.attrelid = i.indexrelid AND bc.relname = \'' . $table . '\' ORDER BY index_name, tab_name';

		}

		/* same as in parent - no need to redifine
		function TableLock($tablename,$lockmode=_OPNSQL_TABLOCK_ACCESS_EXCLUSIVE) {
		$t=$tablename;
		$t=$lockmode;
		return '';
		}
		*/

		function TableUnLock ($tablename) {

			$t = $tablename;
			return '';

		}

		function GetDBType ($datatype, $length = 0, $default = null, $null = false, $decimal = 0) {

			global $opnConfig;
			if ( (isset ($opnConfig['return_tableinfo_as_array']) ) && ($opnConfig['return_tableinfo_as_array'] === TRUE) ) {
				if ($decimal == 0) {
					$decimal = null;
				}
				return $opnConfig['opnSQL']->BuildField ('', $length, $datatype, ! $null, !is_null ($default), $default, $decimal);
			}
			$hlp = '';
			switch ($datatype) {
				case _OPNSQL_TINYINT:
				case _OPNSQL_SMALLINT:
					$hlp .= 'SMALLINT';
					break;
				case _OPNSQL_MEDIUMINT:
				case _OPNSQL_INT:
					$hlp .= 'INTEGER';
					break;
				case _OPNSQL_BIGINT:
					$hlp .= 'INT8';
					break;
				case _OPNSQL_FLOAT:
					$hlp .= 'REAL';
					break;
				case _OPNSQL_DOUBLE:
					$hlp .= 'FLOAT8';
					break;
				case _OPNSQL_NUMERIC:
					$hlp .= 'NUMERIC (' . $length . ',' . $decimal . ')';
					break;
				case _OPNSQL_DATE:
					$hlp .= 'DATE';
					break;
				case _OPNSQL_DATETIME:
					$hlp .= 'DATETIME';
					break;
				case _OPNSQL_TIMESTAMP:
					$hlp .= 'TIMESTAMP';
					break;
				case _OPNSQL_TIME:
					$hlp .= 'TIME';
					break;
				case _OPNSQL_CHAR:
					$hlp .= 'CHAR (' . $length . ')';
					break;
				case _OPNSQL_VARCHAR:
					$hlp .= 'VARCHAR (' . $length . ')';
					break;
				case _OPNSQL_TINYBLOB:
				case _OPNSQL_BLOB:
				case _OPNSQL_MEDIUMBLOB:
				case _OPNSQL_BIGBLOB:
					// $hlp.="BYTEA";  // eigentlich aber da 7Bit
					$hlp .= 'TEXT';
					// ich hoffe das geht
					break;
				case _OPNSQL_TINYTEXT:
				case _OPNSQL_TEXT:
				case _OPNSQL_MEDIUMTEXT:
				case _OPNSQL_BIGTEXT:
					$hlp .= 'TEXT';
					break;
			}
			switch ($datatype) {
				case _OPNSQL_DATETIME:
					// DEFAULT CURRENT_TIMESTAMP.
					if ($default !== NULL) {
						$hlp .= ' DEFAULT ';
						if (strlen ($default) == 0) {
							$hlp .= "TEXT ''";
						} else {
							$hlp .= "TEXT '" . $default . "'";
						}
					}
					break;
				case _OPNSQL_DATE:
					if ($default !== NULL) {
						$hlp .= ' DEFAULT ';
						if (strlen ($default) == 0) {
							$hlp .= "TEXT ''";
						} else {
							$hlp .= "TEXT '" . $default . "'";
						}
					}
					if ($null) {
					} else {
						$hlp .= ' NOT NULL';
					}
					break;
				default:
					if ($default !== NULL) {
						$hlp .= ' DEFAULT ';
						if (strlen ($default) == 0) {
							$hlp .= "''";
						} else {
							$hlp .= "'" . $default . "'";
						}
					}
					if ($null) {
					} else {
						$hlp .= ' NOT NULL';
					}
					break;
			}
			return $hlp;

		}

	}
}

?>